@@ -82,6 +82,7 @@ typedef struct
   gboolean uses_default_factory;
   GtkListItemFactory *factory;
   GtkListItemFactory *list_factory;
+  GtkListItemFactory *header_factory;
   GListModel *model;
   GListModel *filter_model;
   GtkSelectionModel *selection;
@@ -89,6 +90,7 @@ typedef struct
   GtkSelectionModel *current_selection;
 
   GtkExpression *expression;
+  GtkStringFilterMatchMode search_match_mode;
 } AdwComboRowPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (AdwComboRow, adw_combo_row, ADW_TYPE_ACTION_ROW)
@@ -99,10 +101,12 @@ enum {
   PROP_SELECTED_ITEM,
   PROP_MODEL,
   PROP_FACTORY,
+  PROP_HEADER_FACTORY,
   PROP_LIST_FACTORY,
   PROP_EXPRESSION,
   PROP_USE_SUBTITLE,
   PROP_ENABLE_SEARCH,
+  PROP_SEARCH_MATCH_MODE,
   LAST_PROP,
 };
 
@@ -178,7 +182,6 @@ model_changed (AdwComboRow *self)
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
   guint n_items = priv->model ? g_list_model_get_n_items (priv->model) : 0;
 
-  gtk_widget_set_sensitive (GTK_WIDGET (self), n_items > 0);
   gtk_widget_set_visible (priv->arrow_box, n_items > 1);
   gtk_list_box_row_set_activatable (GTK_LIST_BOX_ROW (self), n_items > 1);
 }
@@ -215,16 +218,19 @@ static void
 update_filter (AdwComboRow *self)
 {
   AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
-  
-  if (priv->filter_model) {
-      GtkFilter *filter;
 
-      if (priv->expression) 
-        filter = GTK_FILTER (gtk_string_filter_new (gtk_expression_ref (priv->expression)));
-      else 
-        filter = GTK_FILTER (gtk_every_filter_new ());
-      gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (priv->filter_model), filter);
-      g_object_unref (filter);
+  if (priv->filter_model) {
+    GtkFilter *filter;
+
+    if (priv->expression) {
+      filter = GTK_FILTER (gtk_string_filter_new (gtk_expression_ref (priv->expression)));
+      gtk_string_filter_set_match_mode (GTK_STRING_FILTER (filter), priv->search_match_mode);
+    } else {
+      filter = GTK_FILTER (gtk_every_filter_new ());
+    }
+
+    gtk_filter_list_model_set_filter (GTK_FILTER_LIST_MODEL (priv->filter_model), filter);
+    g_object_unref (filter);
   }
 }
 
@@ -431,6 +437,9 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_FACTORY:
     g_value_set_object (value, adw_combo_row_get_factory (self));
     break;
+  case PROP_HEADER_FACTORY:
+    g_value_set_object (value, adw_combo_row_get_header_factory (self));
+    break;
   case PROP_LIST_FACTORY:
     g_value_set_object (value, adw_combo_row_get_list_factory (self));
     break;
@@ -443,6 +452,9 @@ adw_combo_row_get_property (GObject    *object,
   case PROP_ENABLE_SEARCH:
     g_value_set_boolean (value, adw_combo_row_get_enable_search (self));
     break;
+  case PROP_SEARCH_MATCH_MODE:
+    g_value_set_enum (value, adw_combo_row_get_search_match_mode (self));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -469,6 +481,9 @@ adw_combo_row_set_property (GObject      *object,
   case PROP_LIST_FACTORY:
     adw_combo_row_set_list_factory (self, g_value_get_object (value));
     break;
+  case PROP_HEADER_FACTORY:
+    adw_combo_row_set_header_factory (self, g_value_get_object (value));
+    break;
   case PROP_EXPRESSION:
     adw_combo_row_set_expression (self, gtk_value_get_expression (value));
     break;
@@ -478,6 +493,9 @@ adw_combo_row_set_property (GObject      *object,
   case PROP_ENABLE_SEARCH:
     adw_combo_row_set_enable_search (self, g_value_get_boolean (value));
     break;
+  case PROP_SEARCH_MATCH_MODE:
+    adw_combo_row_set_search_match_mode (self, g_value_get_enum (value));
+    break;
   default:
     G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
   }
@@ -504,6 +522,7 @@ adw_combo_row_dispose (GObject *object)
   g_clear_object (&priv->current_selection);
   g_clear_object (&priv->factory);
   g_clear_object (&priv->list_factory);
+  g_clear_object (&priv->header_factory);
   g_clear_object (&priv->filter_model);
 
   g_clear_object (&priv->model);
@@ -555,7 +574,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
   row_class->activate = adw_combo_row_activate;
 
   /**
-   * AdwComboRow:selected: (attributes org.gtk.Property.get=adw_combo_row_get_selected org.gtk.Property.set=adw_combo_row_set_selected)
+   * AdwComboRow:selected:
    *
    * The position of the selected item.
    *
@@ -568,7 +587,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:selected-item: (attributes org.gtk.Property.get=adw_combo_row_get_selected_item)
+   * AdwComboRow:selected-item:
    *
    * The selected item.
    */
@@ -578,7 +597,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * AdwComboRow:model: (attributes org.gtk.Property.get=adw_combo_row_get_model org.gtk.Property.set=adw_combo_row_set_model)
+   * AdwComboRow:model:
    *
    * The model that provides the displayed items.
    */
@@ -588,7 +607,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:factory: (attributes org.gtk.Property.get=adw_combo_row_get_factory org.gtk.Property.set=adw_combo_row_set_factory)
+   * AdwComboRow:factory:
    *
    * Factory for populating list items.
    *
@@ -601,7 +620,19 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:list-factory: (attributes org.gtk.Property.get=adw_combo_row_get_list_factory org.gtk.Property.set=adw_combo_row_set_list_factory)
+   * AdwComboRow:header-factory:
+   *
+   * The factory for creating header widgets for the popup.
+   *
+   * Since: 1.6
+   */
+  props[PROP_HEADER_FACTORY] =
+    g_param_spec_object ("header-factory", NULL, NULL,
+                         GTK_TYPE_LIST_ITEM_FACTORY,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  /**
+   * AdwComboRow:list-factory:
    *
    * The factory for populating list items in the popup.
    *
@@ -613,7 +644,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:expression: (type GtkExpression) (attributes org.gtk.Property.get=adw_combo_row_get_expression org.gtk.Property.set=adw_combo_row_set_expression)
+   * AdwComboRow:expression: (type GtkExpression)
    *
    * An expression used to obtain strings from items.
    *
@@ -630,7 +661,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                                G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:use-subtitle: (attributes org.gtk.Property.get=adw_combo_row_get_use_subtitle org.gtk.Property.set=adw_combo_row_set_use_subtitle)
+   * AdwComboRow:use-subtitle:
    *
    * Whether to use the current value as the subtitle.
    *
@@ -648,7 +679,7 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * AdwComboRow:enable-search: (attributes org.gtk.Property.get=adw_combo_row_get_enable_search org.gtk.Property.set=adw_combo_row_set_enable_search)
+   * AdwComboRow:enable-search:
    *
    * Whether to show a search entry in the popup.
    *
@@ -664,6 +695,19 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
                           FALSE,
                           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwComboRow:search-match-mode:
+   *
+   * The match mode for the search filter.
+   *
+   * Since: 1.6
+   */
+  props[PROP_SEARCH_MATCH_MODE] =
+    g_param_spec_enum  ("search-match-mode", NULL, NULL,
+                        GTK_TYPE_STRING_FILTER_MATCH_MODE,
+                        GTK_STRING_FILTER_MATCH_MODE_PREFIX,
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   g_object_class_install_properties (object_class, LAST_PROP, props);
 
   gtk_widget_class_set_template_from_resource (widget_class,
@@ -684,6 +728,10 @@ adw_combo_row_class_init (AdwComboRowClass *klass)
 static void
 adw_combo_row_init (AdwComboRow *self)
 {
+  AdwComboRowPrivate *priv = adw_combo_row_get_instance_private (self);
+
+  priv->search_match_mode = GTK_STRING_FILTER_MATCH_MODE_PREFIX;
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   adw_preferences_row_set_use_markup (ADW_PREFERENCES_ROW (self), FALSE);
@@ -705,7 +753,7 @@ adw_combo_row_new (void)
 }
 
 /**
- * adw_combo_row_set_selected: (attributes org.gtk.Method.set_property=selected)
+ * adw_combo_row_set_selected:
  * @self: a combo row
  * @position: the position of the item to select, or
  *   [const@Gtk.INVALID_LIST_POSITION]
@@ -732,7 +780,7 @@ adw_combo_row_set_selected (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_selected: (attributes org.gtk.Method.get_property=selected)
+ * adw_combo_row_get_selected:
  * @self: a combo row
  *
  * Gets the position of the selected item.
@@ -756,7 +804,7 @@ adw_combo_row_get_selected (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_selected_item: (attributes org.gtk.Method.get_property=selected-item)
+ * adw_combo_row_get_selected_item:
  * @self: a combo row
  *
  * Gets the selected item.
@@ -779,7 +827,7 @@ adw_combo_row_get_selected_item (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_get_model: (attributes org.gtk.Method.get_property=model)
+ * adw_combo_row_get_model:
  * @self: a combo row
  *
  * Gets the model that provides the displayed items.
@@ -799,7 +847,7 @@ adw_combo_row_get_model (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_model: (attributes org.gtk.Method.set_property=model)
+ * adw_combo_row_set_model:
  * @self: a combo row
  * @model: (nullable) (transfer none): the model to use
  *
@@ -872,7 +920,7 @@ adw_combo_row_set_model (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_factory: (attributes org.gtk.Method.get_property=factory)
+ * adw_combo_row_get_factory:
  * @self: a combo row
  *
  * Gets the factory for populating list items.
@@ -892,7 +940,7 @@ adw_combo_row_get_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_factory: (attributes org.gtk.Method.set_property=factory)
+ * adw_combo_row_set_factory:
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -912,7 +960,57 @@ adw_combo_row_set_factory (AdwComboRow        *self,
 }
 
 /**
- * adw_combo_row_get_list_factory: (attributes org.gtk.Method.get_property=list-factory)
+ * adw_combo_row_get_header_factory:
+ * @self: a combo row
+ *
+ * Gets the factory that's currently used to create header widgets for the popup.
+ *
+ * Returns: (nullable) (transfer none): The factory in use
+ *
+ * Since: 1.6
+ */
+GtkListItemFactory *
+adw_combo_row_get_header_factory (AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), NULL);
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  return priv->header_factory;
+}
+
+/**
+ * adw_combo_row_set_header_factory:
+ * @self: a combo row
+ * @factory: (nullable) (transfer none): the factory to use
+ *
+ * Sets the factory to use for creating header widgets for the popup.
+ *
+ * Since: 1.6
+ */
+void
+adw_combo_row_set_header_factory (AdwComboRow        *self,
+                                  GtkListItemFactory *factory)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_COMBO_ROW (self));
+  g_return_if_fail (factory == NULL || GTK_LIST_ITEM_FACTORY (factory));
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  if (!g_set_object (&priv->header_factory, factory))
+    return;
+
+  gtk_list_view_set_header_factory (GTK_LIST_VIEW (priv->list), priv->header_factory);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HEADER_FACTORY]);
+}
+
+/**
+ * adw_combo_row_get_list_factory:
  * @self: a combo row
  *
  * Gets the factory for populating list items in the popup.
@@ -932,7 +1030,7 @@ adw_combo_row_get_list_factory (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_list_factory: (attributes org.gtk.Method.set_property=list-factory)
+ * adw_combo_row_set_list_factory:
  * @self: a combo row
  * @factory: (nullable) (transfer none): the factory to use
  *
@@ -963,7 +1061,7 @@ adw_combo_row_set_list_factory (AdwComboRow        *self,
 }
 
 /**
- * adw_combo_row_get_expression: (attributes org.gtk.Method.get_property=expression)
+ * adw_combo_row_get_expression:
  * @self: a combo row
  *
  * Gets the expression used to obtain strings from items.
@@ -983,7 +1081,7 @@ adw_combo_row_get_expression (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_expression: (attributes org.gtk.Method.set_property=expression)
+ * adw_combo_row_set_expression:
  * @self: a combo row
  * @expression: (nullable): an expression
  *
@@ -1027,7 +1125,7 @@ adw_combo_row_set_expression (AdwComboRow   *self,
 }
 
 /**
- * adw_combo_row_get_use_subtitle: (attributes org.gtk.Method.get_property=use-subtitle)
+ * adw_combo_row_get_use_subtitle:
  * @self: a combo row
  *
  * Gets whether to use the current value as the subtitle.
@@ -1047,7 +1145,7 @@ adw_combo_row_get_use_subtitle (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_use_subtitle: (attributes org.gtk.Method.set_property=use-subtitle)
+ * adw_combo_row_set_use_subtitle:
  * @self: a combo row
  * @use_subtitle: whether to use the current value as the subtitle
  *
@@ -1085,7 +1183,7 @@ adw_combo_row_set_use_subtitle (AdwComboRow *self,
 }
 
 /**
- * adw_combo_row_get_enable_search: (attributes org.gtk.Method.set_property=enable-search)
+ * adw_combo_row_get_enable_search:
  * @self: a combo row
  *
  * Gets whether search is enabled.
@@ -1112,7 +1210,7 @@ adw_combo_row_get_enable_search (AdwComboRow *self)
 }
 
 /**
- * adw_combo_row_set_enable_search: (attributes org.gtk.Method.set_property=enable-search)
+ * adw_combo_row_set_enable_search:
  * @self: a combo row
  * @enable_search: whether to enable search
  *
@@ -1145,5 +1243,56 @@ adw_combo_row_set_enable_search (AdwComboRow *self,
   gtk_editable_set_text (GTK_EDITABLE (priv->search_entry), "");
   gtk_widget_set_visible (GTK_WIDGET (priv->search_entry), enable_search);
 
-  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_USE_SUBTITLE]);
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ENABLE_SEARCH]);
+}
+
+/**
+ * adw_combo_row_get_search_match_mode:
+ * @self: a combo row
+ *
+ * Returns the match mode that the search filter is using.
+ *
+ * Returns: the match mode of the search filter
+ *
+ * Since: 1.6
+ */
+GtkStringFilterMatchMode
+adw_combo_row_get_search_match_mode (AdwComboRow *self)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_COMBO_ROW (self), GTK_STRING_FILTER_MATCH_MODE_PREFIX);
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  return priv->search_match_mode;
+}
+
+/**
+ * adw_combo_row_set_search_match_mode:
+ * @self: a combo row
+ * @search_match_mode: the new match mode
+ *
+ * Sets the match mode for the search filter.
+ *
+ * Since: 1.6
+ */
+void
+adw_combo_row_set_search_match_mode (AdwComboRow *self,
+                                     GtkStringFilterMatchMode search_match_mode)
+{
+  AdwComboRowPrivate *priv;
+
+  g_return_if_fail (ADW_IS_COMBO_ROW (self));
+
+  priv = adw_combo_row_get_instance_private (self);
+
+  if (priv->search_match_mode == search_match_mode)
+    return;
+
+  priv->search_match_mode = search_match_mode;
+
+  update_filter (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_SEARCH_MATCH_MODE]);
 }
